@@ -94,12 +94,12 @@ const basic & basic::operator=(const basic & other)
 //////////
 
 /** Construct object from archive_node. */
-basic::basic(const archive_node &n, lst &sym_lst) : flags(0)
+basic::basic(const archive_node &n, lst&) : flags(0)
 {
 	// Reconstruct tinfo_key from class name
-	std::string class_name;
-	if (n.find_string("class", class_name))
-		tinfo_key = find_tinfo_key(class_name);
+	std::string found_class_name;
+	if (n.find_string("class", found_class_name))
+		tinfo_key = find_tinfo_key(found_class_name);
 	else
 		throw (std::runtime_error("archive node contains no class name"));
 }
@@ -177,7 +177,7 @@ next_context:
 }
 
 /** Default output to stream. */
-void basic::do_print(const print_context & c, unsigned level) const
+void basic::do_print(const print_context & c, unsigned) const
 {
 	c.s << "[" << class_name() << " object]";
 }
@@ -195,7 +195,7 @@ void basic::do_print_tree(const print_tree & c, unsigned level) const
 }
 
 /** Python parsable output to stream. */
-void basic::do_print_python_repr(const print_python_repr & c, unsigned level) const
+void basic::do_print_python_repr(const print_python_repr & c, unsigned) const
 {
 	c.s << class_name() << "()";
 }
@@ -230,7 +230,7 @@ unsigned basic::precedence() const
 /** Information about the object.
  *
  *  @see class info_flags */
-bool basic::info(unsigned inf) const
+bool basic::info(unsigned) const
 {
 	// all possible properties are false for basic objects
 	return false;
@@ -246,13 +246,13 @@ size_t basic::nops() const
 }
 
 /** Return operand/member at position i. */
-ex basic::op(size_t i) const
+ex basic::op(size_t) const
 {
 	throw(std::range_error(std::string("basic::op(): ") + class_name() + std::string(" has no operands")));
 }
 
 /** Return modifyable operand/member at position i. */
-ex & basic::let_op(size_t i)
+ex & basic::let_op(size_t)
 {
 	ensure_if_modifiable();
 	throw(std::range_error(std::string("basic::let_op(): ") + class_name() + std::string(" has no operands")));
@@ -308,12 +308,12 @@ ex basic::map(map_function & f) const
 	if (num == 0)
 		return *this;
 
-	basic *copy = NULL;
+	basic *copy = nullptr;
 	for (size_t i=0; i<num; i++) {
 		const ex & o = op(i);
 		const ex & n = f(o);
 		if (!are_ex_trivially_equal(o, n)) {
-			if (copy == NULL)
+			if (copy == nullptr)
 				copy = duplicate();
 			copy->let_op(i) = n;
 		}
@@ -377,15 +377,15 @@ ex basic::collect(const ex & s, bool distributed) const
 
 			exmap cmap;
 			cmap[_ex1] = _ex0;
-			for (const_iterator xi=x.begin(); xi!=x.end(); ++xi) {
+                        for (const auto & xelem : x) {
 				ex key = _ex1;
-				ex pre_coeff = *xi;
-				for (lst::const_iterator li=l.begin(); li!=l.end(); ++li) {
-					int cexp = pre_coeff.degree(*li);
-					pre_coeff = pre_coeff.coeff(*li, cexp);
-					key *= pow(*li, cexp);
+                                ex pre_coeff = xelem;
+				for (const auto & lelem : l) {
+					int cexp = pre_coeff.degree(lelem);
+					pre_coeff = pre_coeff.coeff(lelem, cexp);
+					key *= pow(lelem, cexp);
 				}
-				exmap::iterator ci = cmap.find(key);
+				auto ci = cmap.find(key);
 				if (ci != cmap.end())
 					ci->second += pre_coeff;
 				else
@@ -393,8 +393,8 @@ ex basic::collect(const ex & s, bool distributed) const
 			}
 
 			exvector resv;
-			for (exmap::const_iterator mi=cmap.begin(); mi != cmap.end(); ++mi)
-				resv.push_back((mi->first)*(mi->second));
+                        for (const auto & elem : cmap)
+				resv.push_back((elem.first) * (elem.second));
 			return (new add(resv))->setflag(status_flags::dynallocated);
 
 		} else {
@@ -422,7 +422,7 @@ ex basic::collect(const ex & s, bool distributed) const
 }
 
 /** Perform automatic non-interruptive term rewriting rules. */
-ex basic::eval(int level) const
+ex basic::eval(int) const
 {
 	// There is nothing to do for basic objects:
 	return hold();
@@ -528,7 +528,7 @@ ex basic::scalar_mul_indexed(const ex & self, const numeric & other) const
  *  @param v The complete vector of factors
  *  @return true if the contraction was successful, false otherwise
  *  @see ex::simplify_indexed() */
-bool basic::contract_with(exvector::iterator self, exvector::iterator other, exvector & v) const
+bool basic::contract_with(exvector::iterator, exvector::iterator, exvector&) const
 {
 	// Do nothing
 	return false;
@@ -560,9 +560,9 @@ bool basic::match(const ex & pattern, lst & repl_lst) const
 		// Wildcard matches anything, but check whether we already have found
 		// a match for that wildcard first (if so, the earlier match must be
 		// the same expression)
-		for (lst::const_iterator it = repl_lst.begin(); it != repl_lst.end(); ++it) {
-			if (it->op(0).is_equal(pattern))
-				return is_equal(ex_to<basic>(it->op(1)));
+		for (const auto & elem : repl_lst) {
+			if (elem.op(0).is_equal(pattern))
+				return is_equal(ex_to<basic>(elem.op(1)));
 		}
 		repl_lst.append(pattern == *this);
 		return true;
@@ -599,19 +599,14 @@ bool basic::match(const ex & pattern, lst & repl_lst) const
 /** Helper function for subs(). Does not recurse into subexpressions. */
 ex basic::subs_one_level(const exmap & m, unsigned options) const
 {
-	exmap::const_iterator it;
-
 	if (options & subs_options::no_pattern) {
-		ex thisex = *this;
-		it = m.find(thisex);
-		if (it != m.end())
-			return it->second;
-		return thisex;
+		const auto& it = m.find(*this);
+		return (it != m.end())? it->second : *this;
 	} else {
-		for (it = m.begin(); it != m.end(); ++it) {
+                for (const auto & elem : m) {
 			lst repl_lst;
-			if (match(ex_to<basic>(it->first), repl_lst))
-				return it->second.subs(repl_lst, options | subs_options::no_pattern); // avoid infinite recursion when re-substituting the wildcards
+			if (match(ex_to<basic>(elem.first), repl_lst))
+				return elem.second.subs(repl_lst, options | subs_options::no_pattern); // avoid infinite recursion when re-substituting the wildcards
 		}
 	}
 
@@ -758,7 +753,7 @@ bool basic::is_equal_same_type(const basic & other) const
  *  automatically used instead of match_same_type() if nops() == 0.
  *
  *  @see basic::match */
-bool basic::match_same_type(const basic & other) const
+bool basic::match_same_type(const basic &) const
 {
 	// The default is to only consider subexpressions, but not any other
 	// attributes
