@@ -65,8 +65,9 @@ function_options::function_options(std::string const & n, std::string const & tn
 
 function_options::function_options(std::string const & n, unsigned np)
 {
+        static std::string empty;
 	initialize();
-	set_name(n, std::string());
+        set_name(n, empty);
 	nparams = np;
 }
 
@@ -77,11 +78,12 @@ function_options::~function_options()
 
 void function_options::initialize()
 {
-	set_name("unnamed_function", "\\mbox{unnamed}");
+        static std::string s1("unnamed_function"), s2("\\mbox{unnamed}");
+	set_name(s1, s2);
 	nparams = 0;
 	eval_f = real_part_f = imag_part_f = conjugate_f = derivative_f
-		= power_f = series_f = 0;
-	evalf_f = 0;
+		= power_f = series_f = nullptr;
+	evalf_f = nullptr;
 	evalf_params_first = true;
 	apply_chain_rule = true;
 	use_return_type = false;
@@ -103,11 +105,14 @@ void function_options::initialize()
 function_options & function_options::set_name(std::string const & n,
                                               std::string const & tn)
 {
-	name = n;
-	if (tn==std::string())
-		TeX_name = "{\\rm "+name+"}";
+        name.assign(n);
+	if (tn.empty()) {
+		TeX_name.assign("{\\rm ");
+                TeX_name += n;
+                TeX_name.append("}");
+        }
 	else
-		TeX_name = tn;
+	        TeX_name.assign(tn);
 	return *this;
 }
 
@@ -513,7 +518,7 @@ function::function(unsigned ser, const ex & param1, const ex & param2, const ex
 
 // end of generated lines
 
-function::function(unsigned ser, const exprseq & es) : exprseq(es), serial(ser)
+function::function(unsigned ser, exprseq  es) : exprseq(std::move(es)), serial(ser)
 {
 	tinfo_key = &function::tinfo_static;
 
@@ -528,8 +533,8 @@ function::function(unsigned ser, const exvector & v, bool discardable)
 	tinfo_key = &function::tinfo_static;
 }
 
-function::function(unsigned ser, std::auto_ptr<exvector> vp) 
-  : exprseq(vp), serial(ser)
+function::function(unsigned ser, std::unique_ptr<exvector> vp) 
+  : exprseq(std::move(vp)), serial(ser)
 {
 	tinfo_key = &function::tinfo_static;
 }
@@ -577,13 +582,12 @@ function::function(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst)
 	if (n.find_string("name", s)) {
 		unsigned int ser = 0;
 		unsigned int nargs = seq.size();
-		std::vector<function_options>::const_iterator i = registered_functions().begin(), iend = registered_functions().end();
-		while (i != iend) {
-			if (s == i->name && nargs == i->nparams) {
+                for (const auto & elem : registered_functions()) {
+			if (s == elem.name && nargs == elem.nparams) {
 				serial = ser;
 				return;
 			}
-			++i; ++ser;
+			++ser;
 		}
 		// if the name is not already in the registry, we are
 		// unarchiving a SymbolicFunction without any custom methods
@@ -650,7 +654,7 @@ void function::print(const print_context & c, unsigned level) const
 	GINAC_ASSERT(serial<registered_functions().size());
 	// Dynamically dispatch on print_context type
 	const print_context_class_info *pc_info = &c.get_class_info();
-	if (serial >= py_funcs.py_get_ginac_serial()) {
+	if (serial >= static_cast<unsigned>(py_funcs.py_get_ginac_serial())) {
 		//convert arguments to a PyTuple of Expressions
 		PyObject* args = py_funcs.exvector_to_PyTuple(seq);
 
@@ -698,7 +702,7 @@ void function::print(const print_context & c, unsigned level) const
 
 next_context:
 	unsigned id = pc_info->options.get_id();
-	if (id >= pdt.size() || pdt[id] == NULL) {
+	if (id >= pdt.size() || pdt[id] == nullptr) {
 
 		// Method not found, try parent print_context class
 		const print_context_class_info *parent_pc_info = pc_info->get_parent();
@@ -716,8 +720,8 @@ next_context:
 			    << ", nops=" << nops()
 			    << std::endl;
 			unsigned delta_indent = static_cast<const print_tree &>(c).delta_indent;
-			for (size_t i=0; i<seq.size(); ++i)
-				seq[i].print(c, level + delta_indent);
+			for (auto & elem : seq)
+				elem.print(c, level + delta_indent);
 			c.s << std::string(level + delta_indent, ' ') << "=====" << std::endl;
 
 		} else if (is_a<print_csrc>(c)) {
@@ -796,7 +800,7 @@ ex function::eval(int level) const
 		}
 	}
 
-	if (opt.eval_f==0) {
+	if (opt.eval_f==nullptr) {
 		return this->hold();
 	}
 
@@ -866,14 +870,11 @@ ex function::evalf(int level, PyObject* kwds) const
 	else {
 		eseq.reserve(seq.size());
 		--level;
-		exvector::const_iterator it = seq.begin(), itend = seq.end();
-		while (it != itend) {
-			eseq.push_back(it->evalf(level, kwds));
-			++it;
-		}
+                for (const auto & elem : seq)
+			eseq.push_back(elem.evalf(level, kwds));
 	}
 
-	if (opt.evalf_f==0) {
+	if (opt.evalf_f==nullptr) {
 		return function(serial,eseq).hold();
 	}
 	current_serial = serial;
@@ -932,9 +933,9 @@ ex function::thiscontainer(const exvector & v) const
 	return function(serial, v);
 }
 
-ex function::thiscontainer(std::auto_ptr<exvector> vp) const
+ex function::thiscontainer(std::unique_ptr<exvector> vp) const
 {
-	return function(serial, vp);
+	return function(serial, std::move(vp));
 }
 
 /** Implementation of ex::series for functions.
@@ -944,7 +945,7 @@ ex function::series(const relational & r, int order, unsigned options) const
 	GINAC_ASSERT(serial<registered_functions().size());
 	const function_options &opt = registered_functions()[serial];
 
-	if (opt.series_f==0) {
+	if (opt.series_f==nullptr) {
 		return basic::series(r, order);
 	}
 	ex res;
@@ -1047,7 +1048,7 @@ ex function::conjugate() const
 	GINAC_ASSERT(serial<registered_functions().size());
 	const function_options & opt = registered_functions()[serial];
 
-	if (opt.conjugate_f==0) {
+	if (opt.conjugate_f==nullptr) {
 		return conjugate_function(*this).hold();
 	}
 
@@ -1094,7 +1095,7 @@ ex function::real_part() const
 	GINAC_ASSERT(serial<registered_functions().size());
 	const function_options & opt = registered_functions()[serial];
 
-	if (opt.real_part_f==0)
+	if (opt.real_part_f==nullptr)
 		return basic::real_part();
 
 	if (opt.python_func & function_options::real_part_python_f) {
@@ -1138,7 +1139,7 @@ ex function::imag_part() const
 	GINAC_ASSERT(serial<registered_functions().size());
 	const function_options & opt = registered_functions()[serial];
 
-	if (opt.imag_part_f==0)
+	if (opt.imag_part_f==nullptr)
 		return basic::imag_part();
 
 	if (opt.python_func & function_options::imag_part_python_f) {
@@ -1183,8 +1184,8 @@ ex function::imag_part() const
  *  @see ex::diff */
 ex function::derivative(const symbol & s) const
 {
-	ex result;
-
+        ex result;
+        
 	/*
 	if (serial == Order_SERIAL::serial) {
 		// Order Term function only differentiates the argument
@@ -1195,7 +1196,7 @@ ex function::derivative(const symbol & s) const
 
 	// Check if we need to apply chain rule
 	if (!opt.apply_chain_rule) {
-		if (opt.derivative_f == NULL)
+		if (opt.derivative_f == nullptr)
 			throw(std::runtime_error("function::derivative(): custom derivative function must be defined"));
 
 		if (opt.python_func & function_options::derivative_python_f) {
@@ -1217,7 +1218,7 @@ ex function::derivative(const symbol & s) const
 				throw(std::runtime_error("function::derivative(): python function raised exception"));
 			}
 			// convert output Expression to an ex
-			ex result = py_funcs.pyExpression_to_ex(pyresult);
+			result = py_funcs.pyExpression_to_ex(pyresult);
 			Py_DECREF(pyresult);
 			if (PyErr_Occurred()) { 
 				throw(std::runtime_error("function::derivative(): python function (pyExpression_to_ex) raised exception"));
@@ -1353,7 +1354,7 @@ ex function::pderivative(unsigned diff_param) const // partial differentiation
 	const function_options &opt = registered_functions()[serial];
 	
 	// No derivative defined? Then return abstract derivative object
-	if (opt.derivative_f == NULL)
+	if (opt.derivative_f == nullptr)
 		return fderivative(serial, diff_param, seq);
 
 	current_serial = serial;
@@ -1404,7 +1405,7 @@ ex function::power(const ex & power_param) const // power of function
 	const function_options &opt = registered_functions()[serial];
 	
 	// No derivative defined? Then return abstract derivative object
-	if (opt.power_f == NULL)
+	if (opt.power_f == nullptr)
 		return (new GiNaC::power(*this, power_param))->setflag(status_flags::dynallocated |
 	                                               status_flags::evaluated);
 
@@ -1450,7 +1451,7 @@ ex function::power(const ex & power_param) const // power of function
 
 std::vector<function_options> & function::registered_functions()
 {
-	static std::vector<function_options> * rf = new std::vector<function_options>;
+	static auto  rf = new std::vector<function_options>;
 	return *rf;
 }
 
@@ -1469,8 +1470,8 @@ void function::store_remember_table(ex const & result) const
 unsigned function::register_new(function_options const & opt)
 {
 	size_t same_name = 0;
-	for (size_t i=0; i<registered_functions().size(); ++i) {
-		if (registered_functions()[i].name==opt.name) {
+	for (auto & elem : registered_functions()) {
+		if (elem.name==opt.name) {
 			++same_name;
 		}
 	}
@@ -1504,12 +1505,10 @@ unsigned function::register_new(function_options const & opt)
  *  Throws exception if function was not found. */
 unsigned function::find_function(const std::string &name, unsigned nparams)
 {
-	std::vector<function_options>::const_iterator i = function::registered_functions().begin(), end = function::registered_functions().end();
 	unsigned serial = 0;
-	while (i != end) {
-		if (i->get_name() == name && i->get_nparams() == nparams)
+        for (const auto & elem : registered_functions()) {
+		if (elem.get_name() == name && elem.get_nparams() == nparams)
 			return serial;
-		++i;
 		++serial;
 	}
 	throw (std::runtime_error("no function '" + name + "' with " + ToString(nparams) + " parameters defined"));
@@ -1522,20 +1521,43 @@ std::string function::get_name() const
 	return registered_functions()[serial].name;
 }
 
+void function::set_domain(unsigned d)
+{
+        domain = d;
+        iflags.clear();
+        switch (d) {
+                case domain::complex:
+                        break;
+                case domain::real:
+                        iflags.set(info_flags::real, true);
+                        break;
+                case domain::positive:
+                        iflags.set(info_flags::real, true);
+                        iflags.set(info_flags::positive, true);
+                        iflags.set(info_flags::nonnegative, true);
+                        break;
+                case domain::integer:
+                        iflags.set(info_flags::real, true);
+                        iflags.set(info_flags::integer, true);
+                        break;
+        }
+}
+
 bool function::info(unsigned inf) const
 {
-	if (get_serial() == exp_SERIAL::serial)
+        if (serial == exp_SERIAL::serial)
         {
                 const ex& arg = op(0);
                 switch (inf) {
+                case info_flags::nonzero:
+                        return true;
                 case info_flags::real:
-                        return arg.info(info_flags::real);
                 case info_flags::positive:
                         return arg.info(info_flags::real);
                 }
+                return false;
         }
-	else if (get_serial() == log_SERIAL::serial)
-        {
+        else if (serial == log_SERIAL::serial) {
                 const ex& arg = op(0);
                 switch (inf) {
                 case info_flags::real:
@@ -1550,8 +1572,60 @@ bool function::info(unsigned inf) const
                                 arg.info(info_flags::positive) and
                                 ex_to<numeric>(arg) < *_num1_p;
                 }
+                return false;
         }
-        return false;
+        else if (serial == abs_SERIAL::serial) {
+                switch (inf) {
+                case info_flags::real:
+                case info_flags::nonnegative:
+                        return true;
+                case info_flags::nonzero:
+                        return op(0).info(info_flags::nonzero);
+                }
+                return false;
+        }
+        else if (serial == real_part_function_SERIAL::serial) {
+                switch (inf) {
+                case info_flags::real:
+                case info_flags::nonnegative:
+                        return true;
+                case info_flags::nonzero:
+                        return op(0).info(info_flags::nonzero);
+                }
+                return false;
+        }
+        else if (serial == imag_part_function_SERIAL::serial) {
+                switch (inf) {
+                case info_flags::real:
+                case info_flags::nonnegative:
+                        return true;
+                case info_flags::nonzero:
+                        return op(0).info(info_flags::nonzero);
+                }
+                return false;
+        }
+        else if (serial == binomial_SERIAL::serial) {
+                const ex& arg = op(0);
+                switch (inf) {
+                case info_flags::real or info_flags::nonnegative:
+                        return arg.info(inf);
+                case info_flags::integer:
+                        return arg.info(inf) and op(1).info(inf);
+                }
+                return false;
+        }
+        else if (serial == factorial_SERIAL::serial) {
+                const ex& arg = op(0);
+                switch (inf) {
+                case info_flags::real or info_flags::nonnegative:
+                        return arg.info(inf);
+                case info_flags::integer:
+                        return arg.info(inf);
+                }
+                return false;
+        }
+
+	return iflags.get(inf);
 }
 
 bool has_function(const ex & x)
