@@ -37,6 +37,7 @@
 #include <stdexcept>
 #include <sstream>
 #include <string>
+#include <memory>
 
 namespace GiNaC {
 
@@ -91,7 +92,8 @@ static ex exp_eval(const ex & x)
 	}
 	if (PyErr_Occurred()) {
 		PyErr_Clear();
-	} else if (TwoExOverPiI.info(info_flags::integer)) {
+	} else if (is_exactly_a<numeric>(TwoExOverPiI)
+                and TwoExOverPiI.info(info_flags::integer)) {
 		const numeric z = mod(ex_to<numeric>(TwoExOverPiI),*_num4_p);
 		if (z.is_equal(*_num0_p))
 			return _ex1;
@@ -149,11 +151,11 @@ static void exp_print(const ex & arg, const print_context & c,
 	if (!arg.is_equal(*_num1_p)) {
 		c.s<<"^";
 		std::stringstream tstream;
-		print_context *tcontext_p;
+		std::unique_ptr<print_context> tcontext_p;
 		if (latex) {
-			tcontext_p = new print_latex(tstream, c.options);
+			tcontext_p.reset(new print_latex(tstream, c.options));
 		} else {
-			tcontext_p = new print_dflt(tstream, c.options);
+			tcontext_p.reset(new print_dflt(tstream, c.options));
 		}
 		arg.print(*tcontext_p);
 		std::string argstr = tstream.str();
@@ -228,7 +230,7 @@ static ex log_eval(const ex & x)
 		if (x.is_zero())         // log(0) -> infinity
 			//throw(pole_error("log_eval(): log(0)",0));
 			return NegInfinity;
-		if (x.info(info_flags::negative))
+		if (not x.info(info_flags::inexact) and x.info(info_flags::negative))
 			return (log(-x)+I*Pi);
 		if (x.is_equal(_ex1))  // log(1) -> 0
 			return _ex0;
