@@ -156,12 +156,10 @@ typedef std::vector<sym_desc> sym_desc_vec;
 // Add symbol the sym_desc_vec (used internally by get_symbol_stats())
 static void add_symbol(const ex &s, sym_desc_vec &v)
 {
-	sym_desc_vec::const_iterator it = v.begin(), itend = v.end();
-	while (it != itend) {
-		if (it->sym.is_equal(s))  // If it's already in there, don't add it a second time
+	for (const auto& elem : v)
+		if (elem.sym.is_equal(s))  // If it's already in there, don't add it a second time
 			return;
-		++it;
-	}
+
 	sym_desc d;
 	d.sym = s;
 	v.push_back(d);
@@ -196,7 +194,7 @@ static void get_symbol_stats(const ex &a, const ex &b, sym_desc_vec &v)
 {
 	collect_symbols(a.eval(), v);   // eval() to expand assigned symbols
 	collect_symbols(b.eval(), v);
-	sym_desc_vec::iterator it = v.begin(), itend = v.end();
+	auto it = v.begin(), itend = v.end();
 	while (it != itend) {
 		int deg_a = a.degree(it->sym);
 		int deg_b = b.degree(it->sym);
@@ -320,8 +318,8 @@ numeric numeric::integer_content() const
 
 numeric add::integer_content() const
 {
-	epvector::const_iterator it = seq.begin();
-	epvector::const_iterator itend = seq.end();
+	auto it = seq.begin();
+	auto itend = seq.end();
 	numeric c = *_num0_p, l = *_num1_p;
 	while (it != itend) {
 		GINAC_ASSERT(!is_exactly_a<numeric>(it->rest));
@@ -794,10 +792,10 @@ static bool divide_in_z(const ex &a, const ex &b, ex &q, sym_desc_vec::const_ite
 
 	if (is_exactly_a<mul>(b)) {
 		ex qbar = a;
-		for (const_iterator itrb = b.begin(); itrb != b.end(); ++itrb) {
+		for (const auto& elem : b) {
 			sym_desc_vec sym_stats;
-			get_symbol_stats(a, *itrb, sym_stats);
-			if (!divide_in_z(qbar, *itrb, q, sym_stats.begin()))
+			get_symbol_stats(a, elem, sym_stats);
+			if (!divide_in_z(qbar, elem, q, sym_stats.begin()))
 				return false;
 
 			qbar = q;
@@ -949,17 +947,17 @@ ex ex::content(const ex &x) const
 	ex c = e.integer_content();
 	ex r = e / c;
 	int deg = r.degree(x);
-	ex lcoeff = r.coeff(x, deg);
-	if (lcoeff.info(info_flags::integer))
+	ex lcoef = r.coeff(x, deg);
+	if (lcoef.info(info_flags::integer))
 		return c;
 
 	// GCD of all coefficients
 	int ldeg = r.ldegree(x);
 	if (deg == ldeg)
-		return lcoeff * c / lcoeff.unit(x);
+		return lcoef * c / lcoef.unit(x);
 	ex cont = _ex0;
 	for (int i=ldeg; i<=deg; i++)
-		cont = gcd(r.coeff(x, i), cont, NULL, NULL, false);
+		cont = gcd(r.coeff(x, i), cont, nullptr, nullptr, false);
 	return cont * c;
 }
 
@@ -1099,7 +1097,7 @@ static ex sr_gcd(const ex &a, const ex &b, sym_desc_vec::const_iterator var)
 	// Remove content from c and d, to be attached to GCD later
 	ex cont_c = c.content(x);
 	ex cont_d = d.content(x);
-	ex gamma = gcd(cont_c, cont_d, NULL, NULL, false);
+	ex gamma = gcd(cont_c, cont_d, nullptr, nullptr, false);
 	if (ddeg == 0)
 		return gamma;
 	c = c.primpart(x, cont_c);
@@ -1163,8 +1161,8 @@ numeric numeric::max_coefficient() const
 
 numeric add::max_coefficient() const
 {
-	epvector::const_iterator it = seq.begin();
-	epvector::const_iterator itend = seq.end();
+	auto it = seq.begin();
+	auto itend = seq.end();
 	GINAC_ASSERT(is_exactly_a<numeric>(overall_coeff));
 	numeric cur_max = abs(ex_to<numeric>(overall_coeff));
 	while (it != itend) {
@@ -1213,18 +1211,18 @@ ex add::smod(const numeric &xi) const
 {
 	epvector newseq;
 	newseq.reserve(seq.size()+1);
-	epvector::const_iterator it = seq.begin();
-	epvector::const_iterator itend = seq.end();
+	auto it = seq.begin();
+	auto itend = seq.end();
 	while (it != itend) {
 		GINAC_ASSERT(!is_exactly_a<numeric>(it->rest));
-		numeric coeff = GiNaC::smod(ex_to<numeric>(it->coeff), xi);
-		if (!coeff.is_zero())
-			newseq.push_back(expair(it->rest, coeff));
+		numeric num_coeff = GiNaC::smod(ex_to<numeric>(it->coeff), xi);
+		if (!num_coeff.is_zero())
+			newseq.push_back(expair(it->rest, num_coeff));
 		it++;
 	}
 	GINAC_ASSERT(is_exactly_a<numeric>(overall_coeff));
-	numeric coeff = GiNaC::smod(ex_to<numeric>(overall_coeff), xi);
-	return (new add(newseq,coeff))->setflag(status_flags::dynallocated);
+	numeric num_coeff = GiNaC::smod(ex_to<numeric>(overall_coeff), xi);
+	return (new add(newseq, num_coeff))->setflag(status_flags::dynallocated);
 }
 
 ex mul::smod(const numeric &xi) const
@@ -1237,7 +1235,7 @@ ex mul::smod(const numeric &xi) const
 		it++;
 	}
 #endif // def DO_GINAC_ASSERT
-	mul * mulcopyp = new mul(*this);
+	auto  mulcopyp = new mul(*this);
 	GINAC_ASSERT(is_exactly_a<numeric>(overall_coeff));
 	mulcopyp->overall_coeff = GiNaC::smod(ex_to<numeric>(overall_coeff),xi);
 	mulcopyp->clearflag(status_flags::evaluated);
@@ -1615,7 +1613,7 @@ factored_b:
 
 	// The symbol with least degree which is contained in both polynomials
 	// is our main variable
-	sym_desc_vec::iterator vari = sym_stats.begin();
+	auto vari = sym_stats.begin();
 	while ((vari != sym_stats.end()) && 
 	       (((vari->ldeg_b == 0) && (vari->deg_b == 0)) ||
 	        ((vari->ldeg_a == 0) && (vari->deg_a == 0))))
@@ -1634,7 +1632,7 @@ factored_b:
 	// to the end:
 	rotate(sym_stats.begin(), vari, sym_stats.end());
 
-	sym_desc_vec::const_iterator var = sym_stats.begin();
+	auto var = sym_stats.begin();
 	const ex &x = var->sym;
 
 	// Cancel trivial common factor
@@ -1802,11 +1800,8 @@ ex sqrfree(const ex &a, const lst &l)
 	if (l.nops()==0) {
 		sym_desc_vec sdv;
 		get_symbol_stats(a, _ex0, sdv);
-		sym_desc_vec::const_iterator it = sdv.begin(), itend = sdv.end();
-		while (it != itend) {
-			args.append(it->sym);
-			++it;
-		}
+		for (const auto& elem : sdv)
+			args.append(elem.sym);
 	} else {
 		args = l;
 	}
@@ -1829,7 +1824,7 @@ ex sqrfree(const ex &a, const lst &l)
 
 	// recurse down the factors in remaining variables
 	if (newargs.nops()>0) {
-		exvector::iterator i = factors.begin();
+		auto i = factors.begin();
 		while (i != factors.end()) {
 			*i = sqrfree(*i, newargs);
 			++i;
@@ -1838,9 +1833,11 @@ ex sqrfree(const ex &a, const lst &l)
 
 	// Done with recursion, now construct the final result
 	ex result = _ex1;
-	exvector::const_iterator it = factors.begin(), itend = factors.end();
-	for (int p = 1; it!=itend; ++it, ++p)
-		result *= power(*it, p);
+        {
+        int p = 1;
+	for (const auto& elem : factors)
+		result *= power(elem, p++);
+        }
 
 	// Yun's algorithm does not account for constant factors.  (For univariate
 	// polynomials it works only in the monic case.)  We can correct this by
@@ -1949,7 +1946,7 @@ static ex replace_with_symbol(const ex & e, exmap & repl, exmap & rev_lookup)
 	ex e_replaced = e.subs(repl, subs_options::no_pattern);
 
 	// Expression already replaced? Then return the assigned symbol
-	exmap::const_iterator it = rev_lookup.find(e_replaced);
+	auto it = rev_lookup.find(e_replaced);
 	if (it != rev_lookup.end())
 		return it->second;
 
@@ -1973,9 +1970,9 @@ static ex replace_with_symbol(const ex & e, exmap & repl)
 	ex e_replaced = e.subs(repl, subs_options::no_pattern);
 
 	// Expression already replaced? Then return the assigned symbol
-	for (exmap::const_iterator it = repl.begin(); it != repl.end(); ++it)
-		if (it->second.is_equal(e_replaced))
-			return it->first;
+	for (const auto& elem : repl)
+		if (elem.second.is_equal(e_replaced))
+			return elem.first;
 
 	// Otherwise create new symbol and add to list, taking care that the
 	// replacement expression doesn't itself contain symbols from repl,
@@ -2120,7 +2117,7 @@ ex add::normal(exmap & repl, exmap & rev_lookup, int level) const
 	exvector nums, dens;
 	nums.reserve(seq.size()+1);
 	dens.reserve(seq.size()+1);
-	epvector::const_iterator it = seq.begin(), itend = seq.end();
+	auto it = seq.begin(), itend = seq.end();
 	while (it != itend) {
 		ex n = ex_to<basic>(recombine_pair_to_ex(*it)).normal(repl, rev_lookup, level-1);
 		nums.push_back(n.op(0));
@@ -2137,8 +2134,8 @@ ex add::normal(exmap & repl, exmap & rev_lookup, int level) const
 //std::clog << "add::normal uses " << nums.size() << " summands:\n";
 
 	// Add fractions sequentially
-	exvector::const_iterator num_it = nums.begin(), num_itend = nums.end();
-	exvector::const_iterator den_it = dens.begin(), den_itend = dens.end();
+	auto num_it = nums.begin(), num_itend = nums.end();
+	auto den_it = dens.begin(), den_itend = dens.end();
 //std::clog << " num = " << *num_it << ", den = " << *den_it << std::endl;
 	ex num = *num_it++, den = *den_it++;
 	while (num_it != num_itend) {
@@ -2179,7 +2176,7 @@ ex mul::normal(exmap & repl, exmap & rev_lookup, int level) const
 	exvector num; num.reserve(seq.size());
 	exvector den; den.reserve(seq.size());
 	ex n;
-	epvector::const_iterator it = seq.begin(), itend = seq.end();
+	auto it = seq.begin(), itend = seq.end();
 	while (it != itend) {
 		n = ex_to<basic>(recombine_pair_to_ex(*it)).normal(repl, rev_lookup, level-1);
 		num.push_back(n.op(0));
@@ -2258,7 +2255,7 @@ ex power::normal(exmap & repl, exmap & rev_lookup, int level) const
 ex pseries::normal(exmap & repl, exmap & rev_lookup, int level) const
 {
 	epvector newseq;
-	epvector::const_iterator i = seq.begin(), end = seq.end();
+	auto i = seq.begin(), end = seq.end();
 	while (i != end) {
 		ex restexp = i->rest.normal();
 		if (!restexp.is_zero())
@@ -2381,15 +2378,15 @@ ex ex::to_rational(lst & repl_lst) const
 {
 	// Convert lst to exmap
 	exmap m;
-	for (lst::const_iterator it = repl_lst.begin(); it != repl_lst.end(); ++it)
-		m.insert(std::make_pair(it->op(0), it->op(1)));
+	for (const auto & elem : repl_lst)
+		m.insert(std::make_pair(elem.op(0), elem.op(1)));
 
 	ex ret = bp->to_rational(m);
 
 	// Convert exmap back to lst
 	repl_lst.remove_all();
-	for (exmap::const_iterator it = m.begin(); it != m.end(); ++it)
-		repl_lst.append(it->first == it->second);
+	for (const auto& elem : m)
+		repl_lst.append(elem.first == elem.second);
 
 	return ret;
 }
@@ -2404,15 +2401,15 @@ ex ex::to_polynomial(lst & repl_lst) const
 {
 	// Convert lst to exmap
 	exmap m;
-	for (lst::const_iterator it = repl_lst.begin(); it != repl_lst.end(); ++it)
-		m.insert(std::make_pair(it->op(0), it->op(1)));
+	for (const auto & elem : repl_lst)
+		m.insert(std::make_pair(elem.op(0), elem.op(1)));
 
 	ex ret = bp->to_polynomial(m);
 
 	// Convert exmap back to lst
 	repl_lst.remove_all();
-	for (exmap::const_iterator it = m.begin(); it != m.end(); ++it)
-		repl_lst.append(it->first == it->second);
+	for (const auto& elem : m)
+		repl_lst.append(elem.first == elem.second);
 
 	return ret;
 }
@@ -2519,7 +2516,7 @@ ex expairseq::to_rational(exmap & repl) const
 {
 	epvector s;
 	s.reserve(seq.size());
-	epvector::const_iterator i = seq.begin(), end = seq.end();
+	auto i = seq.begin(), end = seq.end();
 	while (i != end) {
 		s.push_back(split_ex_to_pair(recombine_pair_to_ex(*i).to_rational(repl)));
 		++i;
@@ -2537,7 +2534,7 @@ ex expairseq::to_polynomial(exmap & repl) const
 {
 	epvector s;
 	s.reserve(seq.size());
-	epvector::const_iterator i = seq.begin(), end = seq.end();
+	auto i = seq.begin(), end = seq.end();
 	while (i != end) {
 		s.push_back(split_ex_to_pair(recombine_pair_to_ex(*i).to_polynomial(repl)));
 		++i;
