@@ -153,14 +153,14 @@ void power::do_print_dflt(const print_dflt & c, unsigned level) const
 			c.s << '(';
 		// exp function prints as e^a. Printing powers of this can be
 		// confusing, so we add parenthesis if the basis is exp
-		bool exp_parenthesis = is_ex_the_function(basis, exp) &&
-			basis.op(0) != _ex1;
+		bool exp_parenthesis = is_ex_the_function(basis, exp) and
+                        not basis.op(0).is_integer_one();
 		if (exp_parenthesis)
 			c.s << '(';
 		basis.print(c, precedence());
 		if (exp_parenthesis)
 			c.s << ')';
-		if (!(-exponent).is_integer_one()) {
+		if (not (-exponent).is_integer_one()) {
     		        c.s << "^" << expstr;
 		}
 		if (precedence() <= level)
@@ -186,8 +186,8 @@ void power::do_print_latex(const print_latex & c, unsigned level) const
 	} else {
 		// exp function prints as e^a. Printing powers of this can be
 		// confusing, so we add parenthesis if the basis is exp
-		bool base_parenthesis = is_ex_the_function(basis, exp) &&
-			basis.op(0) != _ex1;
+		bool base_parenthesis = is_ex_the_function(basis, exp) and
+			not basis.op(0).is_integer_one();
 
 		if (precedence() <= level)
 			c.s << "{\\left(";
@@ -199,7 +199,7 @@ void power::do_print_latex(const print_latex & c, unsigned level) const
 		if (base_parenthesis)
 			c.s << "\\right)";
 
-		if (!(-exponent).is_integer_one()) {
+		if (not (-exponent).is_integer_one()) {
     		        c.s << "^{";
 			bool exp_parenthesis = is_exactly_a<power>(exponent);
 			if (exp_parenthesis)
@@ -242,7 +242,7 @@ static void print_sym_pow(const print_context & c, const symbol &x, int exp)
 void power::do_print_csrc(const print_csrc & c, unsigned level) const
 {
 	if (is_a<print_csrc_cl_N>(c)) {
-		if (exponent.is_equal(_ex_1)) {
+		if (exponent.is_integer_one()) {
 			c.s << "recip(";
 			basis.print(c);
 			c.s << ')';
@@ -270,7 +270,7 @@ void power::do_print_csrc(const print_csrc & c, unsigned level) const
 		c.s << ')';
 
 	// <expr>^-1 is printed as "1.0/<expr>" or with the recip() function of CLN
-	} else if (exponent.is_equal(_ex_1)) {
+	} else if (exponent.is_integer_one()) {
 		c.s << "1.0/(";
 		basis.print(c);
 		c.s << ')';
@@ -312,16 +312,25 @@ bool power::info(unsigned inf) const
 		case info_flags::rational_function:
 			return exponent.info(info_flags::integer) &&
 			       basis.info(inf);
+                case info_flags::inexact:
+                        return exponent.info(info_flags::inexact) or
+                                basis.info(info_flags::inexact);
 		case info_flags::algebraic:
 			return !exponent.info(info_flags::integer) ||
 			       basis.info(inf);
 		case info_flags::expanded:
 			return (flags & status_flags::expanded);
 		case info_flags::positive:
+                        if (exponent.info(info_flags::even))
+                                return basis.info(info_flags::real);
+                        if (exponent.info(info_flags::odd))
+                                return basis.info(info_flags::positive);
 			return basis.info(info_flags::positive) && exponent.info(info_flags::real);
 		case info_flags::nonnegative:
 			return (basis.info(info_flags::positive) && exponent.info(info_flags::real)) ||
 			       (basis.info(info_flags::real) && exponent.info(info_flags::integer) && exponent.info(info_flags::even));
+		case info_flags::real:
+			return basis.info(inf) && exponent.info(info_flags::integer);
 		case info_flags::has_indices: {
 			if (flags & status_flags::has_indices)
 				return true;
@@ -465,16 +474,16 @@ ex power::eval(int level) const
 	
 	bool basis_is_numerical = false;
 	bool exponent_is_numerical = false;
-	const numeric *num_basis;
-	const numeric *num_exponent;
+	numeric num_basis;
+	numeric num_exponent;
 	
 	if (is_exactly_a<numeric>(ebasis)) {
 		basis_is_numerical = true;
-		num_basis = &ex_to<numeric>(ebasis);
+		num_basis = ex_to<numeric>(ebasis);
 	}
 	if (is_exactly_a<numeric>(eexponent)) {
 		exponent_is_numerical = true;
-		num_exponent = &ex_to<numeric>(eexponent);
+		num_exponent = ex_to<numeric>(eexponent);
 	}
 
 	// ^(\infty, x)
@@ -486,11 +495,12 @@ ex power::eval(int level) const
 			throw(std::domain_error("power::eval(): pow(Infinity, 0) is undefined."));
 		if (eexponent.info(info_flags::negative))
 			return _ex0;
-		if (eexponent.info(info_flags::positive))
+		if (eexponent.info(info_flags::positive)) {
 			if (basis_inf.is_unsigned_infinity())
 				return UnsignedInfinity;
 			else
 				return mul(pow(basis_inf.get_direction(), eexponent), Infinity);
+                }
 		throw(std::domain_error("power::eval(): pow(Infinity, c)"
 					" for constant of undetermined sign is not defined."));
 	}
@@ -504,11 +514,12 @@ ex power::eval(int level) const
 			throw(std::domain_error("power::eval(): pow(f(x), infinity) is not defined."));
 		// x^(c*oo) --> (x^c)^(+oo)
 		const ex abs_base = abs(pow(ebasis, exp_inf.get_direction()));
-		if (abs_base > _ex1) 
+		if (abs_base > _ex1) {
 			if (ebasis.info(info_flags::positive))
 				return Infinity;
 			else
 				return UnsignedInfinity;
+                        }
 		if (abs_base < _ex1) return _ex0;
 		if (abs_base == _ex1)
 			throw(std::domain_error("power::eval(): pow(1, Infinity) is not defined."));
@@ -519,7 +530,7 @@ ex power::eval(int level) const
 	// ^(x,0) -> 1  (0^0 also handled here)
 	if (eexponent.is_zero() && 
 		!(basis_is_numerical && 
-			num_basis->is_parent_pos_char())) {
+			num_basis.is_parent_pos_char())) {
 		return _ex1;
 	}
 	
@@ -529,9 +540,9 @@ ex power::eval(int level) const
 
 	// ^(0,c1) -> 0 or exception  (depending on real value of c1)
 	if (ebasis.is_zero() && exponent_is_numerical) {
-		if ((num_exponent->real()).is_zero())
+		if ((num_exponent.real()).is_zero())
 			throw (std::domain_error("power::eval(): pow(0,I) is undefined"));
-		else if ((num_exponent->real()).is_negative())
+		else if ((num_exponent.real()).is_negative())
 			throw (pole_error("power::eval(): division by zero",1));
 		else
 			return _ex0;
@@ -553,18 +564,18 @@ ex power::eval(int level) const
 		// ^(c1,c2) -> c1^c2  (c1, c2 numeric(),
 		// except if c1,c2 are rational, but c1^c2 is not)
 		if (basis_is_numerical) {
-			const bool basis_is_crational = num_basis->is_crational();
-                        const bool exponent_is_rational = num_exponent->is_rational();
-			const bool exponent_is_crational = exponent_is_rational || num_exponent->is_crational();
+			const bool basis_is_crational = num_basis.is_crational();
+                        const bool exponent_is_rational = num_exponent.is_rational();
+			const bool exponent_is_crational = exponent_is_rational || num_exponent.is_crational();
 			if (!basis_is_crational || !exponent_is_crational) {
 				// return a plain float
-				return (new numeric(num_basis->power(*num_exponent)))->setflag(status_flags::dynallocated |
+				return (new numeric(num_basis.power(num_exponent)))->setflag(status_flags::dynallocated |
 				                                                               status_flags::evaluated |
 				                                                               status_flags::expanded);
 			}
 
 			if (exponent_is_rational) {
-				const numeric res = num_basis->power(*num_exponent);
+				const numeric res = num_basis.power(num_exponent);
 				if (res.is_crational()) {
 					return res;
 				}
@@ -573,8 +584,8 @@ ex power::eval(int level) const
 
 			// ^(c1,n/m) -> *(c1^q,c1^(n/m-q)), 0<(n/m-q)<1, q integer
 			if (basis_is_crational && exponent_is_rational) {
-				const numeric n = num_exponent->numer();
-				const numeric m = num_exponent->denom();
+				const numeric n = num_exponent.numer();
+				const numeric m = num_exponent.denom();
 				numeric r;
 				numeric q = iquo(n, m, r);
 				if (r.is_negative()) {
@@ -582,12 +593,12 @@ ex power::eval(int level) const
 					--q;
 				}
 				if (q.is_zero()) {  // the exponent was in the allowed range 0<(n/m)<1
-					if (num_basis->is_rational()) {
+					if (num_basis.is_rational()) {
 						// call rational_power_parts
 						// for a^b return c,d such that a^b = c*d^b
-						PyObject* basis = num_basis->to_pyobject();
-						PyObject* exponent = num_exponent->to_pyobject();
-						PyObject* restuple = py_funcs.py_rational_power_parts(basis, exponent);
+						PyObject* basis_py = num_basis.to_pyobject();
+						PyObject* exponent_py = num_exponent.to_pyobject();
+						PyObject* restuple = py_funcs.py_rational_power_parts(basis_py, exponent_py);
 						if(!restuple) {
 							throw(std::runtime_error("power::eval, error in rational_power_parts"));
 						}
@@ -598,8 +609,8 @@ ex power::eval(int level) const
 						if (not ppower_equals_one)
 							result = (new mul(result, ppower))->setflag(status_flags::dynallocated | status_flags::evaluated);
 						Py_DECREF(restuple);
-						Py_DECREF(basis);
-						Py_DECREF(exponent);
+						Py_DECREF(basis_py);
+						Py_DECREF(exponent_py);
 						return result;
 					}
 					return this->hold();
@@ -607,14 +618,14 @@ ex power::eval(int level) const
 					// if r == 0, the following else clause causes the power
 					// constructor to be called again with the same parameter
 					// leading to an infinite loop
-					return num_basis->power(q);
+					return num_basis.power(q);
 				} else {
 					// assemble resulting product, but allowing for a re-evaluation,
 					// because otherwise we'll end up with something like
 					//    (7/8)^(4/3)  ->  7/8*(1/2*7^(1/3))
 					// instead of 7/16*7^(1/3).
-					ex prod = power(*num_basis,r.div(m));
-					return prod*power(*num_basis,q);
+					ex prod = power(num_basis, r.div(m));
+					return prod * power(num_basis, q);
 				}
 			}
 		}
@@ -629,18 +640,18 @@ ex power::eval(int level) const
 			if (is_exactly_a<numeric>(sub_exponent)) {
 				const numeric & num_sub_exponent = ex_to<numeric>(sub_exponent);
 				GINAC_ASSERT(num_sub_exponent!=numeric(1));
-				if (num_exponent->is_integer() || (abs(num_sub_exponent) - (*_num1_p)).is_negative() 
-						|| (num_sub_exponent == *_num_1_p && num_exponent->is_positive())) {
-					return power(sub_basis,num_sub_exponent.mul(*num_exponent));
+				if (num_exponent.is_integer() || (abs(num_sub_exponent) - (*_num1_p)).is_negative() 
+						|| (num_sub_exponent == *_num_1_p && num_exponent.is_positive())) {
+					return power(sub_basis,num_sub_exponent.mul(num_exponent));
 				}
 			}
 		}
 	
-		if (num_exponent->is_integer()) {
+		if (num_exponent.is_integer()) {
                         
                         // ^(*(x,y,z),c1) -> *(x^c1,y^c1,z^c1) (c1 integer)
                         if (is_exactly_a<mul>(ebasis)) {
-                                return expand_mul(ex_to<mul>(ebasis), *num_exponent, 0);
+                                return expand_mul(ex_to<mul>(ebasis), num_exponent, 0);
                         }
 
                         // (2*x + 6*y)^(-4) -> 1/16*(x + 3*y)^(-4)
@@ -657,19 +668,19 @@ ex power::eval(int level) const
 
                                 if (canonicalizable && (icont != *_num1_p)) {
                                         const add& addref = ex_to<add>(ebasis);
-                                        add* addp = new add(addref);
+                                        auto  addp = new add(addref);
                                         addp->setflag(status_flags::dynallocated);
                                         addp->clearflag(status_flags::hash_calculated);
                                         addp->overall_coeff = ex_to<numeric>(addp->overall_coeff).div_dyn(icont);
                                         addp->seq_sorted.resize(0);
-                                        for (epvector::iterator i = addp->seq.begin(); i != addp->seq.end(); ++i)
-                                                i->coeff = ex_to<numeric>(i->coeff).div_dyn(icont);
+                                        for (auto & elem : addp->seq)
+                                                elem.coeff = ex_to<numeric>(elem.coeff).div_dyn(icont);
 
-                                        const numeric c = icont.power(*num_exponent);
+                                        const numeric c = icont.power(num_exponent);
                                         if (likely(c != *_num1_p))
-                                                return (new mul(power(*addp, *num_exponent), c))->setflag(status_flags::dynallocated);
+                                                return (new mul(power(*addp, num_exponent), c))->setflag(status_flags::dynallocated);
                                         else
-                                                return power(*addp, *num_exponent);
+                                                return power(*addp, num_exponent);
                                 }
                         }
                 }
@@ -677,31 +688,31 @@ ex power::eval(int level) const
 		// ^(*(...,x;c1),c2) -> *(^(*(...,x;1),c2),c1^c2)  (c1, c2 numeric(), c1>0)
 		// ^(*(...,x;c1),c2) -> *(^(*(...,x;-1),c2),(-c1)^c2)  (c1, c2 numeric(), c1<0)
 		if (is_exactly_a<mul>(ebasis)) {
-			GINAC_ASSERT(!num_exponent->is_integer()); // should have been handled above
+			GINAC_ASSERT(!num_exponent.is_integer()); // should have been handled above
 			const mul & mulref = ex_to<mul>(ebasis);
 			if (!mulref.overall_coeff.is_equal(_ex1)) {
 				const numeric & num_coeff = ex_to<numeric>(mulref.overall_coeff);
 				if (num_coeff.is_real()) {
 					if (num_coeff.is_positive()) {
-						mul *mulp = new mul(mulref);
+						auto mulp = new mul(mulref);
 						mulp->overall_coeff = _ex1;
 						mulp->setflag(status_flags::dynallocated);
 						mulp->clearflag(status_flags::evaluated);
 						mulp->clearflag(status_flags::hash_calculated);
 						mulp->seq_sorted.resize(0);
 						return (new mul(power(*mulp,exponent),
-						                power(num_coeff,*num_exponent)))->setflag(status_flags::dynallocated);
+						                power(num_coeff, num_exponent)))->setflag(status_flags::dynallocated);
 					} else {
 						GINAC_ASSERT(num_coeff.compare(*_num0_p)<0);
 						if (!num_coeff.is_equal(*_num_1_p)) {
-							mul *mulp = new mul(mulref);
+							auto mulp = new mul(mulref);
 							mulp->overall_coeff = _ex_1;
 							mulp->setflag(status_flags::dynallocated);
 							mulp->clearflag(status_flags::evaluated);
 							mulp->clearflag(status_flags::hash_calculated);
 							mulp->seq_sorted.resize(0);
-							return (new mul(power(*mulp,exponent),
-							                power(abs(num_coeff),*num_exponent)))->setflag(status_flags::dynallocated);
+							return (new mul(power(*mulp, exponent),
+							                power(abs(num_coeff), num_exponent)))->setflag(status_flags::dynallocated);
 						}
 					}
 				}
@@ -710,9 +721,9 @@ ex power::eval(int level) const
 
 		// ^(nc,c1) -> ncmul(nc,nc,...) (c1 positive integer, unless nc is a matrix)
 		if (ebasis.return_type() != return_types::commutative &&
-                    num_exponent->is_pos_integer() &&
+                    num_exponent.is_pos_integer() &&
                     !is_exactly_a<matrix>(ebasis)) {
-			return ncmul(exvector(num_exponent->to_int(), ebasis), true);
+			return ncmul(exvector(num_exponent.to_int(), ebasis), true);
 		}
 	}
 	
@@ -749,7 +760,7 @@ ex power::evalm() const
 {
 	const ex ebasis = basis.evalm();
 	const ex eexponent = exponent.evalm();
-	if (is_exactly_a<matrix>(ebasis)) {
+	if (is_a<matrix>(ebasis)) {
 		if (is_exactly_a<numeric>(eexponent)) {
 			return (new matrix(ex_to<matrix>(ebasis).pow(eexponent)))->setflag(status_flags::dynallocated);
 		}
@@ -761,21 +772,19 @@ bool power::has(const ex & other, unsigned options) const
 {
 	if (!(options & has_options::algebraic))
 		return basic::has(other, options);
-	if (!is_exactly_a<power>(other))
+	if (!is_a<power>(other))
 		return basic::has(other, options);
 	if (!exponent.info(info_flags::integer)
 			|| !other.op(1).info(info_flags::integer))
 		return basic::has(other, options);
 	if (exponent.info(info_flags::posint)
 			&& other.op(1).info(info_flags::posint)
-			&& ex_to<numeric>(exponent).to_int()
-					> ex_to<numeric>(other.op(1)).to_int()
+			&& ex_to<numeric>(exponent)	> ex_to<numeric>(other.op(1))
 			&& basis.match(other.op(0)))
 		return true;
 	if (exponent.info(info_flags::negint)
 			&& other.op(1).info(info_flags::negint)
-			&& ex_to<numeric>(exponent).to_int()
-					< ex_to<numeric>(other.op(1)).to_int()
+			&& ex_to<numeric>(exponent) < ex_to<numeric>(other.op(1))
 			&& basis.match(other.op(0)))
 		return true;
 	return basic::has(other, options);
@@ -796,11 +805,14 @@ ex power::subs(const exmap & m, unsigned options) const
 	if (!(options & subs_options::algebraic))
 		return subs_one_level(m, options);
 
-	for (exmap::const_iterator it = m.begin(); it != m.end(); ++it) {
+        for (const auto & elem : m) {
 		int nummatches = std::numeric_limits<int>::max();
 		lst repls;
-		if (tryfactsubs(*this, it->first, nummatches, repls))
-			return (ex_to<basic>((*this) * power(it->second.subs(ex(repls), subs_options::no_pattern) / it->first.subs(ex(repls), subs_options::no_pattern), nummatches))).subs_one_level(m, options);
+		if (tryfactsubs(*this, elem.first, nummatches, repls))
+			return (ex_to<basic>((*this) * power(elem.second.subs(ex(repls),
+                                subs_options::no_pattern) / elem.first.subs(ex(repls),
+                                subs_options::no_pattern),
+                                nummatches))).subs_one_level(m, options);
 	}
 
 	return subs_one_level(m, options);
@@ -838,7 +850,9 @@ ex power::real_part() const
 		ex basis_real = basis.real_part();
 		if (basis_real == basis)
 			return *this;
-		realsymbol a("a"),b("b");
+		symbol a("a"),b("b");
+                a.set_domain(domain::real);
+                b.set_domain(domain::real);
 		ex result;
 		if (exponent.info(info_flags::posint))
 			result = power(a+I*b,exponent);
@@ -863,7 +877,9 @@ ex power::imag_part() const
 		ex basis_real = basis.real_part();
 		if (basis_real == basis)
 			return 0;
-		realsymbol a("a"),b("b");
+		symbol a("a"),b("b");
+                a.set_domain(domain::real);
+                b.set_domain(domain::real);
 		ex result;
 		if (exponent.info(info_flags::posint))
 			result = power(a+I*b,exponent);
@@ -890,7 +906,7 @@ ex power::imag_part() const
  *  @see ex::diff */
 ex power::derivative(const symbol & s) const
 {
-	if (is_exactly_a<numeric>(exponent)) {
+	if (is_a<numeric>(exponent)) {
 		// D(b^r) = r * b^(r-1) * D(b) (faster than the formula below)
 		epvector newseq;
 		newseq.reserve(2);
@@ -938,7 +954,7 @@ tinfo_t power::return_type_tinfo() const
 
 ex power::expand(unsigned options) const
 {
-	if (is_exactly_a<symbol>(basis) && exponent.info(info_flags::integer)) {
+	if (is_a<symbol>(basis) && exponent.info(info_flags::integer)) {
 		// A special case worth optimizing.
 		setflag(status_flags::expanded);
 		return *this;
@@ -952,37 +968,34 @@ ex power::expand(unsigned options) const
 		epvector powseq;
 		prodseq.reserve(m.seq.size() + 1);
 		powseq.reserve(m.seq.size() + 1);
-		epvector::const_iterator last = m.seq.end();
-		epvector::const_iterator cit = m.seq.begin();
 		bool possign = true;
 
 		// search for positive/negative factors
-		while (cit!=last) {
-			ex e=m.recombine_pair_to_ex(*cit);
+                for (const auto & elem : m.seq) {
+			const ex& e = m.recombine_pair_to_ex(elem);
 			if (e.info(info_flags::positive))
 				prodseq.push_back(pow(e, exponent).expand(options));
 			else if (e.info(info_flags::negative)) {
 				prodseq.push_back(pow(-e, exponent).expand(options));
 				possign = !possign;
 			} else
-				powseq.push_back(*cit);
-			++cit;
+				powseq.push_back(elem);
 		}
 
 		// take care on the numeric coefficient
-		ex coeff=(possign? _ex1 : _ex_1);
+		ex coef = (possign? _ex1 : _ex_1);
 		if (m.overall_coeff.info(info_flags::positive) && m.overall_coeff != _ex1)
 			prodseq.push_back(power(m.overall_coeff, exponent));
 		else if (m.overall_coeff.info(info_flags::negative) && m.overall_coeff != _ex_1)
 			prodseq.push_back(power(-m.overall_coeff, exponent));
 		else
-			coeff *= m.overall_coeff;
+			coef *= m.overall_coeff;
 
 		// If positive/negative factors are found, then extract them.
 		// In either case we set a flag to avoid the second run on a part
 		// which does not have positive/negative terms.
 		if (prodseq.size() > 0) {
-			ex newbasis = coeff*mul(powseq);
+			ex newbasis = coef*mul(powseq);
 			ex_to<basic>(newbasis).setflag(status_flags::purely_indefinite);
 			return ((new mul(prodseq))->setflag(status_flags::dynallocated)*(new power(newbasis, exponent))->setflag(status_flags::dynallocated).expand(options)).expand(options);
 		} else
@@ -997,12 +1010,8 @@ ex power::expand(unsigned options) const
 		const add &a = ex_to<add>(expanded_exponent);
 		exvector distrseq;
 		distrseq.reserve(a.seq.size() + 1);
-		epvector::const_iterator last = a.seq.end();
-		epvector::const_iterator cit = a.seq.begin();
-		while (cit!=last) {
-			distrseq.push_back(power(expanded_basis, a.recombine_pair_to_ex(*cit)));
-			++cit;
-		}
+                for (const auto & elem : a.seq)
+			distrseq.push_back(power(expanded_basis, a.recombine_pair_to_ex(elem)));
 		
 		// Make sure that e.g. (x+y)^(2+a) expands the (x+y)^2 factor
 		if (ex_to<numeric>(a.overall_coeff).is_integer()) {
@@ -1076,7 +1085,6 @@ ex power::expand_add(const add & a, int n, unsigned options) const
 	intvector k(m-1);
 	intvector k_cum(m-1); // k_cum[l]:=sum(i=0,l,k[l]);
 	intvector upper_limit(m-1);
-	int l;
 
 	for (size_t l=0; l<m-1; ++l) {
 		k[l] = 0;
@@ -1087,7 +1095,7 @@ ex power::expand_add(const add & a, int n, unsigned options) const
 	while (true) {
 		exvector term;
 		term.reserve(m+1);
-		for (l=0; l<m-1; ++l) {
+		for (size_t l=0; l<m-1; ++l) {
 			const ex & b = a.op(l);
 			GINAC_ASSERT(!is_exactly_a<add>(b));
 			GINAC_ASSERT(!is_exactly_a<power>(b) ||
@@ -1102,7 +1110,7 @@ ex power::expand_add(const add & a, int n, unsigned options) const
 				term.push_back(power(b,k[l]));
 		}
 
-		const ex & b = a.op(l);
+		const ex & b = a.op(m-1);
 		GINAC_ASSERT(!is_exactly_a<add>(b));
 		GINAC_ASSERT(!is_exactly_a<power>(b) ||
 		             !is_exactly_a<numeric>(ex_to<power>(b).exponent) ||
@@ -1117,7 +1125,7 @@ ex power::expand_add(const add & a, int n, unsigned options) const
 
 
 		numeric f = py_funcs.py_binomial_int(n,k[0]);
-		for (l=1; l<m-1; ++l)
+		for (size_t l=1; l<m-1; ++l)
 		  f *= py_funcs.py_binomial_int(n-k_cum[l-1], k[l]);
 
 		term.push_back(f);
@@ -1125,7 +1133,7 @@ ex power::expand_add(const add & a, int n, unsigned options) const
 		result.push_back(ex((new mul(term))->setflag(status_flags::dynallocated)).expand(options));
 
 		// increment k[]
-		l = m-2;
+		int l = m-2;
 		while ((l>=0) && ((++k[l])>upper_limit[l])) {
 			k[l] = 0;
 			--l;
@@ -1154,11 +1162,11 @@ ex power::expand_add_2(const add & a, unsigned options) const
 	epvector sum;
 	size_t a_nops = a.nops();
 	sum.reserve((a_nops*(a_nops+1))/2);
-	epvector::const_iterator last = a.seq.end();
+	auto last = a.seq.end();
 
 	// power(+(x,...,z;c),2)=power(+(x,...,z;0),2)+2*c*+(x,...,z;0)+c*c
 	// first part: ignore overall_coeff and expand other terms
-	for (epvector::const_iterator cit0=a.seq.begin(); cit0!=last; ++cit0) {
+	for (auto cit0=a.seq.begin(); cit0!=last; ++cit0) {
 		const ex & r = cit0->rest;
 		const ex & c = cit0->coeff;
 		
@@ -1188,23 +1196,20 @@ ex power::expand_add_2(const add & a, unsigned options) const
 			}
 		}
 
-		for (epvector::const_iterator cit1=cit0+1; cit1!=last; ++cit1) {
+		for (auto cit1=cit0+1; cit1!=last; ++cit1) {
 			const ex & r1 = cit1->rest;
 			const ex & c1 = cit1->coeff;
-			sum.push_back(a.combine_ex_with_coeff_to_pair((new mul(r,r1))->setflag(status_flags::dynallocated),
+			sum.push_back(a.combine_ex_with_coeff_to_pair(mul(r,r1).expand(options),
 			                                              _num2_p->mul(ex_to<numeric>(c)).mul_dyn(ex_to<numeric>(c1))));
 		}
 	}
 	
 	GINAC_ASSERT(sum.size()==(a.seq.size()*(a.seq.size()+1))/2);
 	
-	// second part: add terms coming from overall_factor (if != 0)
+	// second part: add terms coming from overall_coeff (if != 0)
 	if (!a.overall_coeff.is_zero()) {
-		epvector::const_iterator i = a.seq.begin(), end = a.seq.end();
-		while (i != end) {
-			sum.push_back(a.combine_pair_with_coeff_to_pair(*i, ex_to<numeric>(a.overall_coeff).mul_dyn(*_num2_p)));
-			++i;
-		}
+                for (const auto & elem : a.seq)
+			sum.push_back(a.combine_pair_with_coeff_to_pair(elem, ex_to<numeric>(a.overall_coeff).mul_dyn(*_num2_p)));
 		sum.push_back(expair(ex_to<numeric>(a.overall_coeff).power_dyn(*_num2_p),_ex1));
 	}
 	
@@ -1243,17 +1248,14 @@ ex power::expand_mul(const mul & m, const numeric & n, unsigned options, bool fr
 	distrseq.reserve(m.seq.size());
 	bool need_reexpand = false;
 
-	epvector::const_iterator last = m.seq.end();
-	epvector::const_iterator cit = m.seq.begin();
-	while (cit!=last) {
-		expair p = m.combine_pair_with_coeff_to_pair(*cit, n);
-		if (from_expand && is_exactly_a<add>(cit->rest) && ex_to<numeric>(p.coeff).is_pos_integer()) {
+        for (const auto & elem : m.seq) {
+		expair p = m.combine_pair_with_coeff_to_pair(elem, n);
+		if (from_expand && is_exactly_a<add>(elem.rest) && ex_to<numeric>(p.coeff).is_pos_integer()) {
 			// this happens when e.g. (a+b)^(1/2) gets squared and
 			// the resulting product needs to be reexpanded
 			need_reexpand = true;
 		}
 		distrseq.push_back(p);
-		++cit;
 	}
 
 	const mul & result = static_cast<const mul &>((new mul(distrseq, ex_to<numeric>(m.overall_coeff).power_dyn(n)))->setflag(status_flags::dynallocated));
