@@ -68,7 +68,7 @@ pseries::pseries() : inherited(&pseries::tinfo_static) { }
  *  @param rel_  expansion variable and point (must hold a relational)
  *  @param ops_  vector of {coefficient, power} pairs (coefficient must not be zero)
  *  @return newly constructed pseries */
-pseries::pseries(const ex &rel_, const epvector &ops_) : basic(&pseries::tinfo_static), seq(ops_)
+pseries::pseries(const ex &rel_, epvector ops_) : basic(&pseries::tinfo_static), seq(std::move(ops_))
 {
 	GINAC_ASSERT(is_exactly_a<relational>(rel_));
 	GINAC_ASSERT(is_exactly_a<symbol>(rel_.lhs()));
@@ -83,17 +83,17 @@ pseries::pseries(const ex &rel_, const epvector &ops_) : basic(&pseries::tinfo_s
 
 pseries::pseries(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst)
 {
-	archive_node::archive_node_cit first = n.find_first("coeff");
-	archive_node::archive_node_cit last = n.find_last("power");
+	auto first = n.find_first("coeff");
+	auto last = n.find_last("power");
 	++last;
 	seq.reserve((last-first)/2);
 
-	for (archive_node::archive_node_cit loc = first; loc < last;) {
+	for (auto loc = first; loc < last;) {
 		ex rest;
-		ex coeff;
+		ex coef;
 		n.find_ex_by_loc(loc++, rest, sym_lst);
-		n.find_ex_by_loc(loc++, coeff, sym_lst);
-		seq.push_back(expair(rest, coeff));
+		n.find_ex_by_loc(loc++, coef, sym_lst);
+		seq.push_back(expair(rest, coef));
 	}
 
 	n.find_ex("var", var, sym_lst);
@@ -103,11 +103,9 @@ pseries::pseries(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst)
 void pseries::archive(archive_node &n) const
 {
 	inherited::archive(n);
-	epvector::const_iterator i = seq.begin(), iend = seq.end();
-	while (i != iend) {
-		n.add_ex("coeff", i->rest);
-		n.add_ex("power", i->coeff);
-		++i;
+        for (const auto & elem : seq) {
+		n.add_ex("coeff", elem.rest);
+		n.add_ex("power", elem.coeff);
 	}
 	n.add_ex("var", var);
 	n.add_ex("point", point);
@@ -129,7 +127,7 @@ void pseries::print_series(const print_context & c, const char *openbrace, const
 	if (seq.empty())
 		c.s << '0';
 
-	epvector::const_iterator i = seq.begin(), end = seq.end();
+	auto i = seq.begin(), end = seq.end();
 	while (i != end) {
 
 		// print a sign, if needed
@@ -248,7 +246,7 @@ int pseries::compare_same_type(const basic & other) const
 		return cmpval;
 	
 	// ...and if that failed the individual elements
-	epvector::const_iterator it = seq.begin(), o_it = o.seq.begin();
+	auto it = seq.begin(), o_it = o.seq.begin();
 	while (it!=seq.end() && o_it!=o.seq.end()) {
 		cmpval = it->compare(*o_it);
 		if (cmpval)
@@ -290,15 +288,14 @@ int pseries::degree(const ex &s) const
 		else
 			return 0;
 	} else {
-		epvector::const_iterator it = seq.begin(), itend = seq.end();
-		if (it == itend)
+                
+		if (seq.size() == 0)
 			return 0;
 		int max_pow = std::numeric_limits<int>::min();
-		while (it != itend) {
-			int pow = it->rest.degree(s);
+                for (const auto & elem : seq) {
+			int pow = elem.rest.degree(s);
 			if (pow > max_pow)
 				max_pow = pow;
-			++it;
 		}
 		return max_pow;
 	}
@@ -318,15 +315,13 @@ int pseries::ldegree(const ex &s) const
 		else
 			return 0;
 	} else {
-		epvector::const_iterator it = seq.begin(), itend = seq.end();
-		if (it == itend)
+		if (seq.size() == 0)
 			return 0;
 		int min_pow = std::numeric_limits<int>::max();
-		while (it != itend) {
-			int pow = it->rest.ldegree(s);
+                for (const auto & elem : seq) {
+			int pow = elem.rest.ldegree(s);
 			if (pow < min_pow)
 				min_pow = pow;
-			++it;
 		}
 		return min_pow;
 	}
@@ -388,11 +383,8 @@ ex pseries::eval(int level) const
 	// Construct a new series with evaluated coefficients
 	epvector new_seq;
 	new_seq.reserve(seq.size());
-	epvector::const_iterator it = seq.begin(), itend = seq.end();
-	while (it != itend) {
-		new_seq.push_back(expair(it->rest.eval(level-1), it->coeff));
-		++it;
-	}
+        for (const auto & elem : seq)
+		new_seq.push_back(expair(elem.rest.eval(level-1), elem.coeff));
 	return (new pseries(relational(var,point), new_seq))->setflag(status_flags::dynallocated | status_flags::evaluated);
 }
 
@@ -408,12 +400,9 @@ ex pseries::evalf(int level, PyObject* parent) const
 	// Construct a new series with evaluated coefficients
 	epvector new_seq;
 	new_seq.reserve(seq.size());
-	epvector::const_iterator it = seq.begin(), itend = seq.end();
-	while (it != itend) {
-		new_seq.push_back(expair(it->rest.evalf(level-1, parent),
-					it->coeff));
-		++it;
-	}
+        for (const auto & elem : seq)
+		new_seq.push_back(expair(elem.rest.evalf(level-1, parent),
+					elem.coeff));
 	return (new pseries(relational(var,point), new_seq))->setflag(status_flags::dynallocated | status_flags::evaluated);
 }
 
@@ -422,7 +411,7 @@ ex pseries::conjugate() const
 	if(!var.info(info_flags::real))
 		return conjugate_function(*this).hold();
 
-	epvector * newseq = conjugateepvector(seq);
+	std::unique_ptr<epvector> newseq(conjugateepvector(seq));
 	ex newpoint = point.conjugate();
 
 	if (!newseq && are_ex_trivially_equal(point, newpoint)) {
@@ -430,7 +419,6 @@ ex pseries::conjugate() const
 	}
 
 	ex result = (new pseries(var==newpoint, newseq ? *newseq : seq))->setflag(status_flags::dynallocated);
-	delete newseq;
 	return result;
 }
 
@@ -444,8 +432,8 @@ ex pseries::real_part() const
 
 	epvector v;
 	v.reserve(seq.size());
-	for(epvector::const_iterator i=seq.begin(); i!=seq.end(); ++i)
-		v.push_back(expair((i->rest).real_part(), i->coeff));
+	for(const auto & elem : seq)
+		v.push_back(expair((elem.rest).real_part(), elem.coeff));
 	return (new pseries(var==point, v))->setflag(status_flags::dynallocated);
 }
 
@@ -459,15 +447,15 @@ ex pseries::imag_part() const
 
 	epvector v;
 	v.reserve(seq.size());
-	for(epvector::const_iterator i=seq.begin(); i!=seq.end(); ++i)
-		v.push_back(expair((i->rest).imag_part(), i->coeff));
+	for(const auto & elem : seq)
+		v.push_back(expair((elem.rest).imag_part(), elem.coeff));
 	return (new pseries(var==point, v))->setflag(status_flags::dynallocated);
 }
 
 ex pseries::eval_integ() const
 {
-	epvector *newseq = NULL;
-	for (epvector::const_iterator i=seq.begin(); i!=seq.end(); ++i) {
+	epvector *newseq = nullptr;
+	for (auto i=seq.begin(); i!=seq.end(); ++i) {
 		if (newseq) {
 			newseq->push_back(expair(i->rest.eval_integ(), i->coeff));
 			continue;
@@ -476,7 +464,7 @@ ex pseries::eval_integ() const
 		if (!are_ex_trivially_equal(newterm, i->rest)) {
 			newseq = new epvector;
 			newseq->reserve(seq.size());
-			for (epvector::const_iterator j=seq.begin(); j!=i; ++j)
+			for (auto j=seq.begin(); j!=i; ++j)
 				newseq->push_back(*j);
 			newseq->push_back(expair(newterm, i->coeff));
 		}
@@ -494,7 +482,7 @@ ex pseries::evalm() const
 	// evalm each coefficient
 	epvector newseq;
 	bool something_changed = false;
-	for (epvector::const_iterator i=seq.begin(); i!=seq.end(); ++i) {
+	for (auto i=seq.begin(); i!=seq.end(); ++i) {
 		if (something_changed) {
 			ex newcoeff = i->rest.evalm();
 			if (!newcoeff.is_zero())
@@ -529,11 +517,8 @@ ex pseries::subs(const exmap & m, unsigned options) const
 	// expansion point
 	epvector newseq;
 	newseq.reserve(seq.size());
-	epvector::const_iterator it = seq.begin(), itend = seq.end();
-	while (it != itend) {
-		newseq.push_back(expair(it->rest.subs(m, options), it->coeff));
-		++it;
-	}
+        for (const auto & elem : seq)
+		newseq.push_back(expair(elem.rest.subs(m, options), elem.coeff));
 	return (new pseries(relational(var,point.subs(m, options)), newseq))->setflag(status_flags::dynallocated);
 }
 
@@ -542,12 +527,10 @@ ex pseries::subs(const exmap & m, unsigned options) const
 ex pseries::expand(unsigned options) const
 {
 	epvector newseq;
-	epvector::const_iterator i = seq.begin(), end = seq.end();
-	while (i != end) {
-		ex restexp = i->rest.expand();
+        for (const auto & elem : seq) {
+		ex restexp = elem.rest.expand();
 		if (!restexp.is_zero())
-			newseq.push_back(expair(restexp, i->coeff));
-		++i;
+			newseq.push_back(expair(restexp, elem.coeff));
 	}
 	return (new pseries(relational(var,point), newseq))
 	        ->setflag(status_flags::dynallocated | (options == 0 ? status_flags::expanded : 0));
@@ -558,33 +541,30 @@ ex pseries::expand(unsigned options) const
 ex pseries::derivative(const symbol & s) const
 {
 	epvector new_seq;
-	epvector::const_iterator it = seq.begin(), itend = seq.end();
 
 	if (s == var) {
 		
 		// FIXME: coeff might depend on var
-		while (it != itend) {
-			if (is_order_function(it->rest)) {
-				new_seq.push_back(expair(it->rest, it->coeff - 1));
+                for (const auto & elem : seq) {
+			if (is_order_function(elem.rest)) {
+				new_seq.push_back(expair(elem.rest, elem.coeff - 1));
 			} else {
-				ex c = it->rest * it->coeff;
+				const ex& c = elem.rest * elem.coeff;
 				if (!c.is_zero())
-					new_seq.push_back(expair(c, it->coeff - 1));
+					new_seq.push_back(expair(c, elem.coeff - 1));
 			}
-			++it;
 		}
 
 	} else {
 
-		while (it != itend) {
-			if (is_order_function(it->rest)) {
-				new_seq.push_back(*it);
+                for (const auto & elem : seq) {
+			if (is_order_function(elem.rest)) {
+				new_seq.push_back(elem);
 			} else {
-				ex c = it->rest.diff(s);
+				const ex& c = elem.rest.diff(s);
 				if (!c.is_zero())
-					new_seq.push_back(expair(c, it->coeff));
+					new_seq.push_back(expair(c, elem.coeff));
 			}
-			++it;
 		}
 	}
 
@@ -594,15 +574,13 @@ ex pseries::derivative(const symbol & s) const
 ex pseries::convert_to_poly(bool no_order) const
 {
 	ex e;
-	epvector::const_iterator it = seq.begin(), itend = seq.end();
-	
-	while (it != itend) {
-		if (is_order_function(it->rest)) {
+        for (const auto & elem : seq) {
+		if (is_order_function(elem.rest)) {
 			if (!no_order)
-				e += Order(power(var - point, it->coeff));
-		} else
-			e += it->rest * power(var - point, it->coeff);
-		++it;
+				e += Order(power(var - point, elem.coeff));
+		}
+                else
+			e += elem.rest * power(var - point, elem.coeff);
 	}
 	return e;
 }
@@ -647,10 +625,9 @@ ex basic::series(const relational & r, int order, unsigned options) const
 	// do Taylor expansion
 	numeric fac = 1;
 	ex deriv = *this;
-	ex coeff = deriv.subs(r, subs_options::no_pattern);
-
-	if (!coeff.is_zero()) {
-		seq.push_back(expair(coeff, _ex0));
+	const ex co = deriv.subs(r, subs_options::no_pattern);
+	if (!co.is_zero()) {
+		seq.push_back(expair(co, _ex0));
 	}
 
 	int n;
@@ -663,9 +640,9 @@ ex basic::series(const relational & r, int order, unsigned options) const
 		if (deriv.is_zero())  // Series terminates
 			return pseries(r, seq);
 
-		coeff = deriv.subs(r, subs_options::no_pattern);
-		if (!coeff.is_zero())
-			seq.push_back(expair(fac.inverse() * coeff, n));
+		const ex coef = deriv.subs(r, subs_options::no_pattern);
+		if (!coef.is_zero())
+			seq.push_back(expair(fac.inverse() * coef, n));
 	}
 	
 	// Higher-order terms, if present
@@ -714,10 +691,10 @@ ex pseries::add_series(const pseries &other) const
 	
 	// Series addition
 	epvector new_seq;
-	epvector::const_iterator a = seq.begin();
-	epvector::const_iterator b = other.seq.begin();
-	epvector::const_iterator a_end = seq.end();
-	epvector::const_iterator b_end = other.seq.end();
+	auto a = seq.begin();
+	auto b = other.seq.begin();
+	auto a_end = seq.end();
+	auto b_end = other.seq.end();
 	int pow_a = std::numeric_limits<int>::max(), pow_b = std::numeric_limits<int>::max();
 	for (;;) {
 		// If a is empty, fill up with elements from b and stop
@@ -782,19 +759,17 @@ ex add::series(const relational & r, int order, unsigned options) const
 	acc = overall_coeff.series(r, order, options);
 	
 	// Add remaining terms
-	epvector::const_iterator it = seq.begin();
-	epvector::const_iterator itend = seq.end();
-	for (; it!=itend; ++it) {
-		ex op;
-		if (is_exactly_a<pseries>(it->rest))
-			op = it->rest;
+	for (const auto & elem : seq) {
+		ex term;
+		if (is_exactly_a<pseries>(elem.rest))
+			term = elem.rest;
 		else
-			op = it->rest.series(r, order, options);
-		if (!it->coeff.is_equal(_ex1))
-			op = ex_to<pseries>(op).mul_const(ex_to<numeric>(it->coeff));
+			term = elem.rest.series(r, order, options);
+		if (!elem.coeff.is_equal(_ex1))
+			term = ex_to<pseries>(term).mul_const(ex_to<numeric>(elem.coeff));
 		
 		// Series addition
-		acc = ex_to<pseries>(acc).add_series(ex_to<pseries>(op));
+		acc = ex_to<pseries>(acc).add_series(ex_to<pseries>(term));
 	}
 	return acc;
 }
@@ -810,13 +785,11 @@ ex pseries::mul_const(const numeric &other) const
 	epvector new_seq;
 	new_seq.reserve(seq.size());
 	
-	epvector::const_iterator it = seq.begin(), itend = seq.end();
-	while (it != itend) {
-		if (!is_order_function(it->rest))
-			new_seq.push_back(expair(it->rest * other, it->coeff));
+        for (const auto & elem : seq) {
+		if (!is_order_function(elem.rest))
+			new_seq.push_back(expair(elem.rest * other, elem.coeff));
 		else
-			new_seq.push_back(*it);
-		++it;
+			new_seq.push_back(elem);
 	}
 	return pseries(relational(var,point), new_seq);
 }
@@ -894,20 +867,18 @@ ex mul::series(const relational & r, int order, unsigned options) const
 	std::vector<bool> ldegree_redo;
 
 	// find minimal degrees
-	const epvector::const_iterator itbeg = seq.begin();
-	const epvector::const_iterator itend = seq.end();
 	// first round: obtain a bound up to which minimal degrees have to be
 	// considered
-	for (epvector::const_iterator it=itbeg; it!=itend; ++it) {
+        for (const auto & elem : seq) {
 
-		ex expon = it->coeff;
+		ex expon = elem.coeff;
 		int factor = 1;
 		ex buf;
 		if (expon.info(info_flags::integer)) {
-			buf = it->rest;
+			buf = elem.rest;
 			factor = ex_to<numeric>(expon).to_int();
 		} else {
-			buf = recombine_pair_to_ex(*it);
+			buf = recombine_pair_to_ex(elem);
 		}
 
 		int real_ldegree = 0;
@@ -944,16 +915,16 @@ ex mul::series(const relational & r, int order, unsigned options) const
 	// method.
 	// here we can ignore ldegrees larger than degbound
 	size_t j = 0;
-	for (epvector::const_iterator it=itbeg; it!=itend; ++it) {
+        for (const auto & elem : seq) {
 		if ( ldegree_redo[j] ) {
-			ex expon = it->coeff;
+			ex expon = elem.coeff;
 			int factor = 1;
 			ex buf;
 			if (expon.info(info_flags::integer)) {
-				buf = it->rest;
+				buf = elem.rest;
 				factor = ex_to<numeric>(expon).to_int();
 			} else {
-				buf = recombine_pair_to_ex(*it);
+				buf = recombine_pair_to_ex(elem);
 			}
 			int real_ldegree = 0;
 			int orderloop = 0;
@@ -977,17 +948,17 @@ ex mul::series(const relational & r, int order, unsigned options) const
 	}
 
 	// Multiply with remaining terms
-	std::vector<int>::const_iterator itd = ldegrees.begin();
-	for (epvector::const_iterator it=itbeg; it!=itend; ++it, ++itd) {
+	auto itd = ldegrees.begin();
+	for (auto it = seq.begin(); it != seq.end(); ++it, ++itd) {
 
 		// do series expansion with adjusted order
-		ex op = recombine_pair_to_ex(*it).series(r, order-degsum+(*itd), options);
+		ex term = recombine_pair_to_ex(*it).series(r, order-degsum+(*itd), options);
 
 		// Series multiplication
-		if (it == itbeg)
-			acc = ex_to<pseries>(op);
+		if (it == seq.begin())
+			acc = ex_to<pseries>(term);
 		else
-			acc = ex_to<pseries>(acc.mul_series(ex_to<pseries>(op)));
+			acc = ex_to<pseries>(acc.mul_series(ex_to<pseries>(term)));
 	}
 
 	return acc.mul_const(ex_to<numeric>(overall_coeff));
@@ -1089,11 +1060,8 @@ ex pseries::power_const(const numeric &p, int deg) const
 pseries pseries::shift_exponents(int deg) const
 {
 	epvector newseq = seq;
-	epvector::iterator i = newseq.begin(), end  = newseq.end();
-	while (i != end) {
-		i->coeff += deg;
-		++i;
-	}
+        for (auto & elem : newseq)
+		elem.coeff += deg;
 	return pseries(relational(var, point), newseq);
 }
 
@@ -1209,15 +1177,13 @@ ex pseries::series(const relational & r, int order, unsigned options) const
 			return *this;
 		else {
 			epvector new_seq;
-			epvector::const_iterator it = seq.begin(), itend = seq.end();
-			while (it != itend) {
-				int o = ex_to<numeric>(it->coeff).to_int();
+                        for (const auto & elem : seq) {
+				int o = ex_to<numeric>(elem.coeff).to_int();
 				if (o >= order) {
 					new_seq.push_back(expair(Order(_ex1), o));
 					break;
 				}
-				new_seq.push_back(*it);
-				++it;
+				new_seq.push_back(elem);
 			}
 			return pseries(r, new_seq);
 		}
